/**
 * Nfc中一些函数的封装
 */
const kDefaultNfcKey = [0xD3,0xF7,0xD3,0xF7,0xD3,0xF7];
/**
 * NFC标签的UID
 */
var gNfcUid = [];
var callback = null;

const NfcUtil = {
  // 测试扫描NFC标签
  nfcStartDiscovery(action) {
    // 获取NFC适配器
    let adapter = tt.getNFCAdapter();
    adapter.startDiscovery({
      success(res) {
        if (!action) {
          tt.showModal({ content: JSON.stringify(res) });
        }
      },
      fail(err) {
        tt.showModal({ content: JSON.stringify(err) });
      }
    });

    // 启动扫描之后开始执行相关行为事件
    action && action(adapter);
  },

  createCallback(action, adapter) {
    callback = (res) => {
      gNfcUid = Array.from(new Uint8Array(res.uid));
      if (action) {
        console.log('result res=', res, gNfcUid);
        action(adapter);
      } else {
        console.log('result res=', res, gNfcUid);
        tt.showModal({ content: JSON.stringify(gNfcUid) });
      }
    };
  },

  onDiscovered(action) {
    let that = this;
    // 调用这个函数来监听NFC标签
    this.nfcStartDiscovery((adapter) => {
      // 注销callback
      adapter.offDiscovered(callback);
      that.createCallback(action, adapter);
      // 重新注册callback
      adapter.onDiscovered(callback);
    });
  },
  nfcAConnect(action) {
    this.onDiscovered((adapter) => {
      //  获取NfcA对象
      let nfca = adapter.getNfcA();
      nfca.connect({
        success(res) {
          if (action) {
            console.log('connect res=' + JSON.stringify(res));
            action(nfca);
          } else {
            tt.showModal({ content: JSON.stringify(res) });
            NfcUtil.nfcClose(nfca);
          }
        }, 
        fail(err) {
          tt.showModal({ content: JSON.stringify(err) });
          NfcUtil.nfcClose(nfca);
        }
      });
    });
  },
  // 测试获取NFC-A atqa数据
  nfcGetAtqa() {
    this.nfcAConnect((nfca) => {
      nfca.getAtqa({
        success(res) {
          console.log('getAtqa success res=', res, Array.from(new Uint8Array(res.data)));
          tt.showModal({ content: JSON.stringify(Array.from(new Uint8Array(res.data))) });
          NfcUtil.nfcClose(nfca);
        },
        fail(err) {
          NfcUtil.nfcClose(nfca);
        }
      })
    });
  },
  nfcGetSak() {
    this.nfcAConnect((nfca) => {
      nfca.getSak({
        success(res) {
          tt.showModal({ content: JSON.stringify(res) });
          NfcUtil.nfcClose(nfca);
        },
        fail(err) {
          NfcUtil.nfcClose(nfca);
        }
      })
    });
  },
  
  nfcTransceive(action) {
    this.nfcAConnect((nfca) => {
      let data = NfcUtil.nfcTestAuthData(gNfcUid);
      nfca.transceive({
        data: new Uint8Array(data).buffer,
        success(res) {
          let result = JSON.stringify(Array.from(new Uint8Array(res.data)));
          if (result.length == 0) {
            tt.showModal({ content: '结果为空，认证失败' });
            NfcUtil.nfcClose(nfca);
            return;
          }
          // result.length > 0表示认证通过
          if (action) {
            action(nfca);
          } else {
            // 弹出传输结果
            let content = '认证数据: ' + JSON.stringify(data) + '\n'
                      + '认证结果: ' + JSON.stringify(Array.from(new Uint8Array(res.data))) + '\n';
            tt.showModal({ content });
            NfcUtil.nfcClose(nfca);
          }
        },
        fail(err) {
          NfcUtil.nfcClose(nfca);
        }
      })
    });
  },

  nfcTransceiveRead() {
    let data = [0x30, this.sectorToBlock(1)];
    this.nfcTransceive((nfca) => {
      nfca.transceive({
        data: new Uint8Array(data).buffer,
        success(res) {
          // 弹出传输结果
          let content = '读指令: ' + JSON.stringify(data) + '\n'
                    + '读数据: ' + JSON.stringify(Array.from(new Uint8Array(res.data))) + '\n';
          tt.showModal({ content });
          NfcUtil.nfcClose(nfca);
        },
        fail(err) {
          NfcUtil.nfcClose(nfca);
        }
      });
    });
  },
  nfcTransceiveWrite() {
    let cmd = [0xA0]; // MF write command
    cmd.push(this.sectorToBlock(1)); // 选择扇区，测试使用1号扇区
    // 数据必须是16位数据
    cmd.push.apply(cmd, [
      0x55, 0xff, 0x11, 0x21, 0x33,
      0x55, 0xff, 0x11, 0x21, 0x33, 
      0x55, 0xff, 0x11, 0x21, 0x33, 0x12
    ]);
    
    this.nfcTransceive((nfca) => {
      nfca.transceive({
        data: new Uint8Array(cmd).buffer,
        success(res) {
          // 弹出传输结果
          let content = '写入数据: ' + JSON.stringify(cmd) + '\n'
                    + '返回结果: ' + JSON.stringify(Array.from(new Uint8Array(res.data))) + '\n';
          tt.showModal({ content });
          NfcUtil.nfcClose(nfca);
        },
        fail(err) {
          NfcUtil.nfcClose(nfca);
        }
      });
    });
  },
  nfcGetMaxTransceiveLength() {
    this.nfcAConnect((nfca) => {
      nfca.getMaxTransceiveLength({
        success(res) {
          // 弹出传输结果
          tt.showModal({ content: JSON.stringify(res) });
          NfcUtil.nfcClose(nfca);
        },
        fail(err) {
          NfcUtil.nfcClose(nfca);
        }
      })
    });
  },
  /**
   * close nfc
   * @param {*} adapter 
   */
  nfcClose(nfc) {
    nfc.close({
      success(res) {
        console.log(JSON.stringify(res));
      },
      fail(res) {
        console.log(JSON.stringify(res));
      }
    })
  },
  /**
   * Return the first block of a given sector.
   * @param {*} sectorIndex 
   */
  sectorToBlock(sectorIndex) {
    if (sectorIndex < 32) {
        return sectorIndex * 4;
    } else {
        return 32 * 4 + (sectorIndex - 32) * 16;
    }
  },
  /**
   * 测试NFC卡认证
   */
  nfcTestAuthData(uid) {
    let cmd = [0x60];
    cmd.push(this.sectorToBlock(1));
    cmd.push.apply(cmd, uid);
    cmd.push.apply(cmd, kDefaultNfcKey);
    return cmd;
  }
}

export default NfcUtil;